@@ -1,5 +1,4 @@
-import six
-from six import ensure_binary, ensure_text
+from six import ensure_text
 import shutil
 import logging
 from ast import literal_eval
@@ -616,6 +615,8 @@ class Manifest(object):
             [self.technical_name]
         )
 
+        po_files = []
+
         for language, module, rows in translations:
             filename = get_translation_filename(language, module)
             trans_path = self.path / 'i18n' / filename
@@ -647,3 +648,7 @@ class Manifest(object):
                 po_writer.add_entries(rows)
 
                 po_writer.write()
+
+                po_files.append(po_writer)
+
+        return po_files
